#!/usr/bin/env python
"""
A generic template

null distribution based on sample permutations



"""

__author__ = "Adam Richards"

class SomeClass(object):
    """
    A generic class
    """

    def __init__(self):
        """
        Constructor

        """










if __name__ == "__main__":
    print "Running..."
