"""
statistics classes and functions

"""


from ClusteringLib import get_silhouette_values
from ClusteringData import dataCircle,dataCircleLabels,dataScatter,dataScatterLabels
from ClusteringData import dataLetters,dataLettersLabels,dataNetwork,dataNetworkLabels
from SpectralClustering import SpectralCluster
from SpectralClusterResults import SpectralClusterResults
